@@ -62,13 +62,33 @@ def isCloser(elem):
     "Are we looking at a closing tag?"
     if isinstance(elem, WmlIterator):
         elem = elem.element
-    return elem.startswith("[/")
+    return type(elem) == type("") and elem.startswith("[/")
+
+def isMacroCloser(elem):
+    "Are we looking at a macro closer?"
+    if isinstance(elem, WmlIterator):
+        elem = elem.element
+    return type(elem) == type("") and elem == closeMacroType
 
 def isOpener(elem):
     "Are we looking at an opening tag?"
     if isinstance(elem, WmlIterator):
         elem = elem.element
-    return elem.startswith("[") and not isCloser(elem)
+    return type(elem) == type("") and elem.startswith("[") and not isCloser(elem)
+
+def isMacroOpener(elem):
+    "Are we looking at a macro opener?"
+    if isinstance(elem, WmlIterator):
+        elem = elem.element
+    return type(elem) == type("") and elem.startswith("{")
+
+def isAttribute(elem):
+    "Are we looking at an attribute (or attribute tuple)?"
+    if isinstance(elem, WmlIterator):
+        elem = elem.element
+    if type(elem) == type(()):
+        elem = elem[0]
+    return type(elem) == type("") and elem.endswith("=")
 
 class WmlIterator(object):
     """Return an iterable WML navigation object.
@@ -116,6 +136,23 @@ Important Attributes:
         begincomment = text.find('#')
         if begincomment < 0:
             begincomment = None
+        beginquote = text[:begincomment].find('<<')
+        while beginquote >= 0:
+            endquote = -1
+            beginofend = beginquote+2
+            while endquote < 0:
+                endquote = text.find('>>', beginofend)
+                if endquote < 0:
+                    if self.lineno + span >= len(lines):
+                        self.printError('reached EOF due to unterminated string at line', self.lineno+1)
+                        return text, span
+                    beginofend = len(text)
+                    text += lines[self.lineno + span]
+                    span += 1
+            begincomment = text.find('#', endquote+2)
+            if begincomment < 0:
+                begincomment = None
+            beginquote = text[:begincomment].find('<<', endquote+2)
         beginquote = text[:begincomment].find('"')
         while beginquote >= 0:
             endquote = -1
@@ -188,10 +225,20 @@ Important Attributes:
             #enddef - closes a scope
         macro calls: "{MACRO_NAME}"
             {MACRO_NAME - opens a scope
-            } - closes a scope (not an element)
+            } - closes a scope
         """
         elements = [] #(elementType, sortPos, scopeDelta)
-        # first remove any quoted strings
+        # first remove any lua strings
+        beginquote = text.find('<<')
+        while beginquote >= 0:
+            endquote = text.find('>>')
+            if endquote < -1:
+                text = text[:beginquote]
+                beginquote = -1 #terminate loop
+            else:
+                text = text[:beginquote] + text[endquote+2:]
+                beginquote = text.find('<<')
+        # remove any quoted strings
         beginquote = text.find('"')
         while beginquote >= 0:
             endquote = text.find('"', beginquote+1)
@@ -220,6 +267,8 @@ Important Attributes:
             return (['#define'],)*2
         elif text.find('#enddef') >= 0:
             elements.append(('#enddef', text.find('#enddef'), -1))
+        elif text.startswith('#po:') or text.startswith('# po:'):
+            elements.append(("#po", 0, 0))
         else:
             commentSearch = 0
         begincomment = text.find('#', commentSearch)
@@ -236,7 +285,7 @@ Important Attributes:
                 if k:
                     elements.append((k+'=', m.start()+i, 0))
         for m in macroOpenPattern.finditer(text):
-            elements.append((m.group(1)+'}', m.start(), 1))
+            elements.append((m.group(1), m.start(), 1))
         for m in macroClosePattern.finditer(text):
             elements.append((closeMacroType, m.start(), -1))
         #sort by start position
@@ -252,8 +301,7 @@ Important Attributes:
             while scopeDelta > 0:
                 openedScopes.append(elem)
                 scopeDelta -= 1
-            if elem != closeMacroType:
-                resultElements.append(elem)
+            resultElements.append(elem)
         return resultElements, openedScopes
 
     def printScopeError(self, elementType):
@@ -355,6 +403,21 @@ Important Attributes:
             raise StopIteration
         return self
 
+    def isOpener(self):
+        return isOpener(self)
+
+    def isCloser(self):
+        return isCloser(self)
+
+    def isMacroOpener(self):
+        return isMacroOpener(self)
+
+    def isMacroCloser(self):
+        return isMacroCloser(self)
+
+    def isAttribute(self):
+        return isAttribute(self)
+
     def iterScope(self):
         """Return an iterator for the current scope"""
         if not self.scopes:
